<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 74dc0c93ba38 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-15 13:23 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Class we_html_table
 *
 * Provides functions for creating html tags used in forms.
 */
class we_html_table extends we_html_baseElement{
	/** @var int */
	private $cols = 0;

	/** @var we_html_thead */
	protected $header;

	/** @var we_html_tbody */
	protected $body;

	/** @var we_html_tfoot */
	protected $footer;

	/**
	 * Constructor of class we_html_table
	 *
	 * @param      $attribs                                array<string,string|string[]>
	 * @param      $rows_num                               int                 (optional)
	 * @param      $cols_num                               int                 (optional)
	 * @param ?we_html_tr[] $content
	 */
	public function __construct(array $attribs = [], int $rows_num = 0, int $cols_num = 0, array $content = null){
		parent::__construct('table', true, $attribs);
		$this->cols = $cols_num;
		$this->body = new we_html_tbody($rows_num, $cols_num, $content);
	}

	public function addRow(we_html_tr $row): self{
		$this->body->addRow($row);
		return $this;
	}

	/**
	 * This function adds a row to the table
	 * @param      $rows_num                               int                 (optional)
	 * @see        we_html_table()
	 * @return     integer
	 */
	public function addRows(int $rows_num = 1, array $attribs = []): int{
		return $this->body->addRows($rows_num, $attribs);
	}

	/**
	 * This function adds a column to the table
	 *
	 * @param      $cols_num                               int                 (optional)
	 * @see        we_html_table()
	 * @return     we_html_table
	 */
	public function addCol(int $cols_num = 1): self{
		$this->cols += $cols_num;
		$this->body->addCol($cols_num);
		return $this;
	}

	public function addHeaderRow(array $cols, array $rowAttr = [], array $headAttr = []): self{
		$childs = [];
		for($j = 0; $j < $this->cols; $j++){
			$childs[] = new we_html_th($cols[$j] ?? '');
		}
		$this->header = new we_html_thead(new we_html_tr($childs, $rowAttr), $headAttr);
		return $this;
	}

	public function addFooterRow(array $cols, array $rowAttr = [], array $footAttr = []): self{
		$childs = [];
		for($j = 0; $j < $this->cols; $j++){
			$childs[] = new we_html_th($cols[$j] ?? '');
		}
		$this->footer = new we_html_tfoot(new we_html_tr($childs, $rowAttr), $footAttr);
		return $this;
	}

	public function setHeader(we_html_thead $header): self{
		$this->header = $header;
		return $this;
	}

	public function setBody(we_html_tbody $body): self{
		$this->body = $body;
		return $this;
	}

	public function setFooter(we_html_tfoot $foot): self{
		$this->footer = $foot;
		return $this;
	}

	public function isEmpty(): bool{
		return $this->header->isEmpty() && $this->body->isEmpty() && $this->footer->isEmpty();
	}

	public function getRowCount(): int{
		return $this->body->getCount();
	}

	/**
	 * This function sets the current column being edited
	 *
	 * @param      $rowid                                  int
	 * @param      $colid                                  int
	 * @param      $content                                string              (optional)
	 * @param      $attribs                                array<string,string|string[]>               (optional)
	 *
	 * @return     we_html_table
	 */
	public function setCol(int $rowid, int $colid, string $content, array $attribs = []): self{
		$this->body->setCol($rowid, $colid, $content, $attribs);
		return $this;
	}

	/**
	 * Assigns the attributes of a column
	 *
	 * @param      int $rowid
	 * @param      array<string,string> $attribs                                (optional)
	 *
	 * @return     we_html_table
	 */
	public function setRowAttributes(int $rowid, array $attribs = []): self{
		$this->body->setRowAttributes($rowid, $attribs);
		return $this;
	}

	/**
	 * Assigns the attributes of a column
	 *
	 * @param      $rowid                                  int
	 * @param      $colid                                  int
	 * @param      $attribs                                array               (optional)
	 *
	 * @return     we_html_table
	 */
	public function setColAttributes(int $rowid, int $colid, array $attribs = []): self{
		$this->body->setColAttributes($rowid, $colid, $attribs);
		return $this;
	}

	public function getHtml(): string{
		$this->content = ($this->header ? $this->header->getHtml() : '') .
			$this->body->getHtml() .
			($this->footer ? $this->footer->getHtml() : '');
		return parent::getHTML();
	}

}
