<?php
declare(strict_types=1);
/* webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Util Functions
 *
 * all functions in this class are static! Please use it in static form:
 * @static
 */
abstract class we_http_util{

	/**
	 *
	 * @param string $server
	 * @param string $url
	 * @param int $port
	 * @param string $username
	 * @param string $password
	 * @param int $timeout
	 * @return string
	 * @throws we_exception_http
	 */
	public static function getHTTP(string $server, string $url, int $port = 0, string $username = '', string $password = '', int $timeout = 0): string{
		if(file_exists(WE_CUSTOM_PATH . 'conf/proxysettings.inc.php')){
			include_once(WE_CUSTOM_PATH . 'conf/proxysettings.inc.php');
		}

//FIXME: check differences to we_xml_browser::getFile
		if(!str_contains($server, '://')){
			$server = 'http' . ($port == 443 ? 's' : '') . '://' . (($username && $password) ? $username . ':' . $password . '@' : '') . $server . ( $port ? ':' . $port : '');
		}
		switch(self::getHttpOption()){
			case 'fopen':
				if(defined('WE_PROXYHOST')){
					$proxyhost = defined('WE_PROXYHOST') ? WE_PROXYHOST : '';
					$proxyport = (defined('WE_PROXYPORT') && WE_PROXYPORT) ? (int) WE_PROXYPORT : 80;
					$proxy_user = defined('WE_PROXYUSER') ? WE_PROXYUSER : '';
					$proxy_pass = defined('WE_PROXYPASSWORD') ? WE_PROXYPASSWORD : '';

					return self::getHttpThroughProxy(($server . $url), $proxyhost, $proxyport, $proxy_user, $proxy_pass, $timeout);
				}
				$ctx = stream_context_create($timeout ? ['http' => ['timeout' => $timeout]] : []);
				//use file_get_contents, it will set $http_response_header too!!
				$page = file_get_contents($server . $url, false, $ctx) ?: '';
				$matches = [];
				if(!empty($http_response_header)){
					preg_match('#HTTP/\d+\.\d+ (\d+)#', $http_response_header[0], $matches);
					$status = (int) ($matches[1] ?: 200);
				} else {
					$status = 500;
				}
				if(!in_array($status, [200, 300, 301, 302, 303, 304, 307, 308])){
					throw new we_exception_http('Server Error: Failed opening URL: ' . $server . $url . ' ' . $status, $status, $http_response_header);
				}
				return $page;
			case 'curl':
				$response = self::getCurlHttp($server, $url, [], false, $timeout);
				$status = (int) ($response['status'] ?: 200);
				if($status != 200){
					throw new we_exception_http($response['error'], $status, $response);
				}
				return $response['data'];
			default:
				throw new we_exception_http('Server error: Unable to open URL (php configuration directive allow_url_fopen=Off)');
		}
	}

	/**
	 * @return string
	 * @param bool $slash
	 * @desc returns the protocol, the webServer is running, http or https, when slash is true - :// is added to protocol
	 */
	public static function getServerProtocol(bool $slash = false): string{
		return (self::isHttps() ? 'https' : 'http') . ($slash ? '://' : '');
	}

	private static function getServerAuth(): string{
		$pwd = rawurlencode(defined('HTTP_USERNAME') ? HTTP_USERNAME : ($_SERVER['PHP_AUTH_USER'] ?? '')) . ':' .
			rawurlencode(defined('HTTP_PASSWORD') ? HTTP_PASSWORD : ($_SERVER['PHP_AUTH_PW'] ?? '')) . '@';
		return (strlen($pwd) > 3) ? $pwd : '';
	}

	public static function getServerUrl(bool $useUserPwd = false): string{
		$port = isset($_SERVER['SERVER_PORT']) && ($port = intval($_SERVER['SERVER_PORT'])) &&
			((self::isHttps() && $port !== 443) || (!self::isHttps() && $port !== 80)) ? ':' . $port : '';
		$auth = self::getServerAuth();
		$pwd = ($useUserPwd && strlen($auth) > 3 ) ? $auth : '';

		return we_http_util::getServerProtocol(true) . $pwd . $_SERVER['SERVER_NAME'] . $port;
	}

	public static function makeQueryString(array $query = [], array $filter = []): string{
		//remove potential Cookies and filter from query
		$mainDoc = we_base_WE::$mainDoc['doc'];
		$filterArr = array_merge([//filter special variables
			'edit_object',
			'edit_document',
			we_tag_write::EDIT_OBJ_PARAM,
			we_tag_write::EDIT_DOC_PARAM,
			'we_transaction',
			'we_cmd',
			'we_cmd[1]',
			'pv_id',
			'pv_tid',
			'jwt',
			'bsuniquevid',
			's'//password-form
			], array_keys($query), array_keys($filter), array_keys($_COOKIE), (TAGLINKS_OBJECTSEOURLS && $mainDoc instanceof we_document_triggerDocument && $mainDoc->Url && show_SeoLinks() ?
			[WEOBJ_PARAM, 'we_oid'] : [])
		);

		if(!empty($_REQUEST)){
			foreach($_REQUEST as $key => $val){
				if((!in_array($key, $filterArr)) && (!str_starts_with($key, 'we_ui_'))){
					$query[$key] = $val;
				}
			}
		}
		return http_build_query($query);
	}

	/**
	 *
	 * @return string
	 */
	public static function getHttpOption(): string{
		if(ini_get('allow_url_fopen') != 1){
			@ini_set('allow_url_fopen', '1');
			return (ini_get('allow_url_fopen') != 1 ?
				(function_exists('curl_init') ? 'curl' : 'none') :
				'fopen');
		}
		return 'fopen';
	}

	/**
	 *
	 * @return bool true if we is called via https
	 */
	public static function isHttps(): bool{
		return isset($_SERVER['HTTPS']) && (strtoupper($_SERVER['HTTPS']) === 'ON' || $_SERVER['HTTPS'] == 1);
	}

	private static function getCurlHttp(string $server, string $path = '', array $files = [], bool $header = false, int $timeout = 0): array{
		$response = [
			'data' => '', // data if successful
			'status' => 0, // 0=ok otherwise error
			'error' => '' // error string
		];
		$parsedurl = parse_url($server);
		$protocol = (isset($parsedurl['scheme']) ?
			$parsedurl['scheme'] . '://' :
			'http://');

		$port = (isset($parsedurl['port']) ? ':' . $parsedurl['port'] : '');
		$pathA = explode('?', $path);
		$url = $protocol . $parsedurl['host'] . $port . $pathA[0];
		if(isset($pathA[1]) && strlen($url . $pathA[1]) < 2000){
//it is safe to have uri's lower than 2k chars - so no need to do a post which servers (e.g. twitter) do not accept.
			$url .= '?' . $pathA[1];
			unset($pathA[1]);
		}
		$params = [];

		$session = curl_init();
		if(!$session){
			return [];
		}
		curl_setopt($session, CURLOPT_URL, $url);
		curl_setopt($session, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($session, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($session, CURLOPT_MAXREDIRS, 5);

		if($timeout){
			curl_setopt($session, CURLOPT_CONNECTTIMEOUT, $timeout);
		}

		/* 	if($username != ''){
		  curl_setopt($session, CURLOPT_USERPWD, $username . ':' . $password);
		  } */

		if(!empty($pathA[1])){
			$url_param = explode('&', $pathA[1]);
			foreach($url_param as $cur){
				$param_split = explode('=', $cur);
				$params[$param_split[0]] = $param_split[1] ?? '';
			}
		}

		if($files){
			foreach($files as $k => $v){
				$params[$k] = '@' . $v;
			}
		}

		if($params){
			curl_setopt($session, CURLOPT_POST, 1);
			curl_setopt($session, CURLOPT_POSTFIELDS, $params);
		}

		if($header){
			curl_setopt($session, CURLOPT_HEADER, 1);
		}

		if(defined('WE_PROXYHOST') && WE_PROXYHOST != ''){

			$proxyhost = defined('WE_PROXYHOST') ? WE_PROXYHOST : '';
			$proxyport = (defined('WE_PROXYPORT') && WE_PROXYPORT) ? WE_PROXYPORT : '80';
			$proxy_user = defined('WE_PROXYUSER') ? WE_PROXYUSER : '';
			$proxy_pass = defined('WE_PROXYPASSWORD') ? WE_PROXYPASSWORD : '';

			if($proxyhost != ''){
				curl_setopt($session, CURLOPT_PROXY, $proxyhost . ':' . $proxyport);
				if($proxy_user != ''){
					curl_setopt($session, CURLOPT_PROXYUSERPWD, $proxy_user . ':' . $proxy_pass);
				}
				curl_setopt($session, CURLOPT_SSL_VERIFYPEER, false);
			}
		}

		$data = curl_exec($session);

		if(curl_errno($session)){
			$info = curl_getinfo($session);
			$response['status'] = empty($info['http_code']) ? 1 : $info['http_code'];
			$response['error'] = curl_error($session);
			return $response;
		}
		$response['status'] = 0;
		$response['data'] = $data;
		curl_close($session);


		return $response;
	}

	private static function getHttpThroughProxy(string $url, string $proxyhost, int $proxyport, string $proxy_user, string $proxy_pass, int $timeout): string{
		$errno = $errstr = '';
		$file = fsockopen($proxyhost, $proxyport, $errno, $errstr, $timeout ?: 30);

		if(!$file){
			return '';
		}
		$ret = '';

		// send headers
		fputs($file, 'GET ' . $url . " HTTP/1.0\r\n");
		fputs($file, "Proxy-Connection: Keep-Alive\r\n");
		fputs($file, 'User-Agent: PHP ' . PHP_VERSION . "\r\n");
		fputs($file, "Pragma: no-cache\r\n");
		if($proxy_user){
			fputs($file, 'Proxy-authorization: Basic ' . base64_encode($proxy_user . ':' . $proxy_pass) . "\r\n");
		}
		fputs($file, "\r\n");

		// write comoplete file and cut http header before returning
		while(!feof($file)){
			$data = fread($file, 8192);
			$ret .= $data;
		}
		fclose($file);

		return str_starts_with($ret, 'HTTP/') ? (substr($ret, (int) strpos($ret, "\r\n\r\n") + 4) ?: '') : $ret;
	}

	public static function getScriptName(bool $param = false): string{
		static $ret = '';
		if(!$ret){
			$docRoot = WE_DOCUMENT_ROOT;
			$weRoot = (realpath($docRoot . '/..' . WEBEDITION_DIR) ?: realpath($docRoot . WEBEDITION_DIR)) . '/';
			$ret = we_base_file::addPreSlash((isset($_SERVER['SCRIPT_FILENAME']) ? strtr(realpath($_SERVER['SCRIPT_FILENAME']) ?: '', [
						$docRoot => '',
						$weRoot => WEBEDITION_DIR,
						'\\' => '/'
					]) : $_SERVER['SCRIPT_NAME'])) .
				($param && !empty($_SERVER['QUERY_STRING']) ? '?' . $_SERVER['QUERY_STRING'] : '');
		}
		return $ret;
	}

}
