<?php
declare(strict_types=1);

/**

 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://framework.zend.com/license/new-bsd
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@zend.com so we can send you a copy immediately.
 *
 * @category   Zend
 * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 * @version    $Id$
 */

/**
 * Class representing a MIME part.
 *
 * @category   Zend
 * @copyright  Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 */
class we_mail_mimePart{
	/**
	 * Type
	 *
	 * @var string
	 */
	public $type = we_mail_mime::TYPE_OCTETSTREAM;

	/**
	 * Encoding
	 *
	 * @var string
	 */
	public $encoding = we_mail_mime::ENCODING_8BIT;

	/**
	 * ID
	 *
	 * @var string
	 */
	public $id;

	/**
	 * Disposition
	 *
	 * @var string
	 */
	public $disposition = '';

	/**
	 * Filename
	 *
	 * @var string
	 */
	public $filename;

	/**
	 * Description
	 *
	 * @var string
	 */
	public $description;

	/**
	 * Character set
	 *
	 * @var string
	 */
	public $charset;

	/**
	 * Boundary
	 *
	 * @var string
	 */
	public $boundary;

	/**
	 * Location
	 *
	 * @var string
	 */
	public $location;

	/**
	 * Language
	 *
	 * @var string
	 */
	public $language;

	/**
	 * Content
	 *
	 * @var mixed
	 */
	protected $content;

	/**
	 * create a new Mime Part.
	 * The (unencoded) content of the Part as passed
	 * as a string or stream
	 *
	 * @param string $content String or Stream containing the content
	 */
	public function __construct(string $content){
		$this->content = $content;
	}

	/**
	 * Get the Content of the current Mime Part in the given encoding.
	 *
	 * @param  string $EOL Line end; defaults to
	 * @return string
	 */
	public function getContent(string $EOL = we_mail_mime::LINEEND): string{
		return we_mail_mime::encode($this->content, $this->encoding, $EOL);
	}

	/**
	 * Create and return the array of headers for this MIME part
	 *
	 * @param  string $EOL Line end; defaults to
	 * @return array
	 */
	public function getHeadersArray(string $EOL = we_mail_mime::LINEEND): array{
		$headers = [
			'Content-Type' => $this->type .
			($this->charset ? '; charset=' . $this->charset : '') .
			($this->boundary ? ';' . $EOL . ' boundary="' . $this->boundary . '"' : '')
		];

		if($this->encoding){
			$headers['Content-Transfer-Encoding'] = $this->encoding;
		}

		if($this->id){
			$headers['Content-ID'] = '<' . $this->id . '>';
		}

		if($this->disposition){
			$headers['Content-Disposition'] = $this->disposition . ($this->filename ? '; ' . $EOL . ' filename="' . $this->filename . '"' : '');
		}

		if($this->description){
			$headers['Content-Description'] = $this->description;
		}

		if($this->location){
			$headers['Content-Location'] = $this->location;
		}

		if($this->language){
			$headers['Content-Language'] = $this->language;
		}

		return $headers;
	}

	/**
	 * Return the headers for this part as a string
	 *
	 * @param  string $EOL Line end; defaults to
	 * @return string
	 */
	public function getHeaders(string $EOL = we_mail_mime::LINEEND): string{
		$res = '';
		foreach($this->getHeadersArray($EOL) as $field => $header){
			$res .= $field . ': ' . $header . $EOL;
		}

		return $res;
	}

	public static function getHTMLPart(string $html, string $charset): self{
		$mp = new self($html);
		$mp->encoding = we_mail_mime::ENCODING_QUOTEDPRINTABLE;
		$mp->type = we_mail_mime::TYPE_HTML;
		$mp->disposition = we_mail_mime::DISPOSITION_INLINE;
		$mp->charset = $charset;
		return $mp;
	}

}
