<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: c564388e074d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-17 13:01 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_shopField extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['name' => false, 'reference' => false, 'shopname' => false], __CLASS__);
		$name = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		$reference = self::getAttribute('reference', $attribs, '', we_base_request::STRING);
		$shopname = self::getAttribute('shopname', $attribs, '', we_base_request::STRING);

		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);

		if($type === 'checkbox'){
			self::attributeMissing($attribs, ['value' => false], __CLASS__);
		}

		$values = self::getAttribute('values', $attribs, [], we_base_request::STRING_LIST); // select, choice
		$value = htmlspecialchars(self::getAttribute('value', $attribs, '', we_base_request::STRING)); // checkbox
		$checked = (bool) self::getAttribute('checked', $attribs, false, we_base_request::BOOL); // checkbox

		if($checked){
			self::attributeMissing($attribs, ['value' => false], __CLASS__);
		}
		$mode = self::getAttribute('mode', $attribs, '', we_base_request::STRING);

		$fieldname = ($reference === 'article' ? we_shop_shop::ARTICLE_CUSTOM_FIELD : we_shop_shop::CART_CUSTOM_FIELD) . '[' . $name . ']';

		$isFieldForCheckBox = false;

		if($reference === 'article'){ // name depends on value
			$orgVal = (!$shopname) ? we_base_request::string(we_shop_shop::ARTICLE_CUSTOM_FIELD, '', $name) : '';
			// does not exist here - we are only in article - custom fields are not stored on documents

			if(we_tag_listview::$current && ($tmpVal = we_weTag_util::tag('field', ['name' => $name]))){
				$orgVal = (string) $tmpVal;
			}
		} else {
			try{
				$basket = we_tag_createShop::getBasket($shopname);
				$orgVal = $basket->getCartField($name);
				$isFieldForCheckBox = $basket->hasCartField($name);
			} catch (we_exception_base $e){
				$orgVal = '';
				$isFieldForCheckBox = false;
			}
		}

		$atts = self::removeAttribs($attribs, ['name', 'reference', 'shopname', 'type', 'values', 'value', 'checked', 'mode']);

		if($type != 'checkbox' && $type != 'choice' && $type != 'radio' && $value){
			// value is compared to saved value in some cases
			// be careful with different behaviour when using value and values
			$orgVal = (string) ($orgVal ?: $value);
		}

		switch($type){
			case 'checkbox':
				return self::getCheckbox($atts, $fieldname, $value, $orgVal, $isFieldForCheckBox, $checked);
			case 'choice':
				return we_html_element::span(['class' => 'we we_shopField'], we_html_tools::inputChoiceField($fieldname, $orgVal, $values, $atts, $mode));
			case 'hidden':
				return self::htmlTag('input', array_merge(self::removeAttribs($atts, ['reference']), [
						'name' => $fieldname,
						'value' => $orgVal,
						'type' => 'hidden',
				]));

			case 'print':
				return self::print($attribs, $orgVal);
			case 'select':
				return we_html_element::span(['class' => 'we we_shopField'], self::selectField($fieldname, $orgVal, $values, $atts, false));
			case 'country':
				$newAtts = self::removeAttribs($attribs, ['name', 'type', 'value', 'values', 'checked', 'mode']);
				$newAtts['name'] = we_shop_shop::CART_CUSTOM_FIELD . '[' . $name . ']';
				$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
				$doc = self::getDocForTag($docAttr);
				$lang = $doc->Language;
				$langcode = we_tag_input::getLangCode($attribs);
				$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));
				foreach($topCountries as $countrykey => &$countryvalue){
					$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
				}
				unset($countryvalue);

				$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
				foreach($shownCountries as $countrykey => &$countryvalue){
					$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
				}
				unset($countryvalue);
				$oldLocale = setlocale(LC_ALL, NULL);
				setlocale(LC_ALL, $lang . '.UTF-8');
				asort($topCountries, SORT_LOCALE_STRING);
				asort($shownCountries, SORT_LOCALE_STRING);
				if($oldLocale){
					setlocale(LC_ALL, $oldLocale);
				}
				$content = '';
				if(WE_COUNTRIES_DEFAULT != ''){
					$content .= '<option value="--" ' . ($orgVal === '--' ? ' selected="selected">' : '>') . WE_COUNTRIES_DEFAULT . '</option>';
				}
				foreach($topCountries as $countrykey => &$countryvalue){
					$content .= '<option value="' . $countrykey . '" ' . ($orgVal == $countrykey ? ' selected="selected">' : '>') . $countryvalue . '</option>';
				}
				unset($countryvalue);

				if(!empty($topCountries) && !empty($shownCountries)){
					$content .= '<option value="-" disabled="disabled">----</option>';
				}
				foreach($shownCountries as $countrykey2 => &$countryvalue2){
					$content .= '<option value="' . $countrykey2 . '" ' . ($orgVal == $countrykey2 ? ' selected="selected">' : '>') . $countryvalue2 . '</option>';
				}
				unset($countryvalue2);

				return we_html_element::span(['class' => 'we we_shopField'], self::htmlTag('select', $newAtts, $content, true));

			case 'language':
				$newAtts = self::removeAttribs($attribs, ['name', 'type', 'value', 'values', 'checked', 'mode']);
				$newAtts['name'] = we_shop_shop::CART_CUSTOM_FIELD . '[' . $name . ']';
				$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
				$doc = self::getDocForTag($docAttr);
				$lang = $doc->Language;
				$langcode = we_tag_input::getLangCode($attribs);

				$frontendL = we_base_WE::$Langs['FrontendLanguages'];
				foreach($frontendL as &$lcvalue){
					$lccode = explode('_', $lcvalue);
					$lcvalue = $lccode[0];
				}
				$frontendLL = [];
				foreach($frontendL as &$lcvalue){
					$frontendLL[$lcvalue] = we_base_country::getTranslation($lcvalue, we_base_country::LANGUAGE, $langcode);
				}

				$oldLocale = setlocale(LC_ALL, NULL);
				setlocale(LC_ALL, $lang . '.UTF-8');
				asort($frontendLL, SORT_LOCALE_STRING);
				if($oldLocale){
					setlocale(LC_ALL, $oldLocale);
				}
				$content = '';
				foreach($frontendLL as $langkey => &$langvalue){
					$content .= '<option value="' . $langkey . '" ' . ($orgVal == $langkey ? ' selected="selected">' : '>') . $langvalue . '</option>' . "\n";
				}
				unset($langvalue);
				return we_html_element::span(['class' => 'we we_shopField'], self::htmlTag('select', $newAtts, $content, true));
			case 'textarea':
				return we_html_element::span(['class' => 'we we_shopField'], self::textareaField($fieldname, $orgVal, $atts));

			case 'radio':
				if($checked && $orgVal === ''){
					$atts['checked'] = 'checked';
				}
				return we_html_element::span(['class' => 'we we_shopField'], self::inputRadioField($fieldname, $orgVal, $value, $atts));

			case 'textinput':
			default:
				return we_html_element::span(['class' => 'we we_shopField'], self::inputTextInputField($fieldname, $orgVal, $atts));
		}
	}

	protected static function getCheckbox(array $atts, string $fieldname, string $value, string $orgVal, bool $isFieldForCheckBox, bool $checked): string{
		$atts = self::removeAttribs($atts, ['size']);
		//$atts['name'] = $fieldname; changed to $tnpname because of new hidden field #6544
		//we_getInputCheckboxField() not possible because sessionField type="checkbox" has a mandatory value
		$tmpname = md5(uniqid(__CLASS__, true)); // #6590, changed from: uniqid(time())
		$atts['name'] = $tmpname;
		$atts['type'] = 'checkbox';
		$atts['value'] = $value;
		$atts['onclick'] = 'this.form.elements[\'' . $fieldname . '\'].value=(this.checked) ? \'' . htmlspecialchars($value) . '\' : \'\''; //#6544
		if(($orgVal == $value) || (!$isFieldForCheckBox) && $checked){
			$atts['checked'] = 'checked';
		}

		return we_html_element::span(['class' => 'we we_shopField'], self::htmlTag('input', $atts) . we_html_element::hidden($fieldname, $orgVal));
	}

	protected static function print(array $attribs, string $orgVal): string{
		$ascountry = self::getAttribute('ascountry', $attribs, false, we_base_request::BOOL);
		$aslanguage = self::getAttribute('aslanguage', $attribs, false, we_base_request::BOOL);
		if($ascountry || $aslanguage){
			$langcode = we_tag_input::getLangCode($attribs);
			return ($ascountry && $orgVal === '--' ? '' : we_base_country::getTranslation($orgVal, ($ascountry ? we_base_country::TERRITORY : we_base_country::LANGUAGE), $langcode));
		}
		return $orgVal;
	}

}
